@@ -22,17 +22,17 @@
 #include "../common/threading_utils.h"
 #include "../data/adapter.h"
 #include "../data/iterative_device_dmatrix.h"
+#include "file_iterator.h"
 
-#if DMLC_ENABLE_STD_THREAD
 #include "./sparse_page_source.h"
 #include "./sparse_page_dmatrix.h"
-#endif  // DMLC_ENABLE_STD_THREAD
 
 namespace dmlc {
 DMLC_REGISTRY_ENABLE(::xgboost::data::SparsePageFormatReg<::xgboost::SparsePage>);
 DMLC_REGISTRY_ENABLE(::xgboost::data::SparsePageFormatReg<::xgboost::CSCPage>);
 DMLC_REGISTRY_ENABLE(::xgboost::data::SparsePageFormatReg<::xgboost::SortedCSCPage>);
 DMLC_REGISTRY_ENABLE(::xgboost::data::SparsePageFormatReg<::xgboost::EllpackPage>);
+DMLC_REGISTRY_ENABLE(::xgboost::data::SparsePageFormatReg<::xgboost::GHistIndexMatrix>);
 }  // namespace dmlc
 
 namespace {
@@ -200,10 +200,10 @@ void LoadFeatureType(std::vector<std::string>const& type_names, std::vector<Feat
       types->emplace_back(FeatureType::kNumerical);
     } else if (elem == "q") {
       types->emplace_back(FeatureType::kNumerical);
-    } else if (elem == "categorical") {
+    } else if (elem == "c") {
       types->emplace_back(FeatureType::kCategorical);
     } else {
-      LOG(FATAL) << "All feature_types must be one of {int, float, i, q, categorical}.";
+      LOG(FATAL) << "All feature_types must be one of {int, float, i, q, c}.";
     }
   }
 }
@@ -361,13 +361,18 @@ void MetaInfo::SetInfo(const char* key, const void* dptr, DataType dtype, size_t
     labels.resize(num);
     DISPATCH_CONST_PTR(dtype, dptr, cast_dptr,
                        std::copy(cast_dptr, cast_dptr + num, labels.begin()));
+    auto valid = std::none_of(labels.cbegin(), labels.cend(), [](auto y) {
+      return std::isnan(y) || std::isinf(y);
+    });
+    CHECK(valid) << "Label contains NaN, infinity or a value too large.";
   } else if (!std::strcmp(key, "weight")) {
     auto& weights = weights_.HostVector();
     weights.resize(num);
     DISPATCH_CONST_PTR(dtype, dptr, cast_dptr,
                        std::copy(cast_dptr, cast_dptr + num, weights.begin()));
-    auto valid = std::all_of(weights.cbegin(), weights.cend(),
-                             [](float w) { return w >= 0; });
+    auto valid = std::none_of(weights.cbegin(), weights.cend(), [](float w) {
+      return w < 0 || std::isinf(w) || std::isnan(w);
+    });
     CHECK(valid) << "Weights must be positive values.";
   } else if (!std::strcmp(key, "base_margin")) {
     auto& base_margin = base_margin_.HostVector();
@@ -420,8 +425,8 @@ void MetaInfo::SetInfo(const char* key, const void* dptr, DataType dtype, size_t
         dtype, dptr, cast_dptr,
         std::copy(cast_dptr, cast_dptr + num, h_feature_weights.begin()));
     bool valid =
-        std::all_of(h_feature_weights.cbegin(), h_feature_weights.cend(),
-                    [](float w) { return w >= 0; });
+        std::none_of(h_feature_weights.cbegin(), h_feature_weights.cend(),
+                     [](float w) { return w < 0; });
     CHECK(valid) << "Feature weight must be greater than 0.";
   } else {
     LOG(FATAL) << "Unknown key for MetaInfo: " << key;
@@ -500,13 +505,17 @@ void MetaInfo::GetFeatureInfo(const char *field,
   }
 }
 
-void MetaInfo::Extend(MetaInfo const& that, bool accumulate_rows) {
+void MetaInfo::Extend(MetaInfo const& that, bool accumulate_rows, bool check_column) {
   if (accumulate_rows) {
     this->num_row_ += that.num_row_;
   }
   if (this->num_col_ != 0) {
-    CHECK_EQ(this->num_col_, that.num_col_)
-        << "Number of columns must be consistent across batches.";
+    if (check_column) {
+      CHECK_EQ(this->num_col_, that.num_col_)
+          << "Number of columns must be consistent across batches.";
+    } else {
+      this->num_col_ = std::max(this->num_col_, that.num_col_);
+    }
   }
   this->num_col_ = that.num_col_;
 
@@ -630,11 +639,34 @@ DMatrix::~DMatrix() {
   }
 }
 
+DMatrix *TryLoadBinary(std::string fname, bool silent) {
+  int magic;
+  std::unique_ptr<dmlc::Stream> fi(
+      dmlc::Stream::Create(fname.c_str(), "r", true));
+  if (fi != nullptr) {
+    common::PeekableInStream is(fi.get());
+    if (is.PeekRead(&magic, sizeof(magic)) == sizeof(magic)) {
+      if (!DMLC_IO_NO_ENDIAN_SWAP) {
+        dmlc::ByteSwap(&magic, sizeof(magic), 1);
+      }
+      if (magic == data::SimpleDMatrix::kMagic) {
+        DMatrix *dmat = new data::SimpleDMatrix(&is);
+        if (!silent) {
+          LOG(CONSOLE) << dmat->Info().num_row_ << 'x' << dmat->Info().num_col_
+                       << " matrix with " << dmat->Info().num_nonzero_
+                       << " entries loaded from " << fname;
+        }
+        return dmat;
+      }
+    }
+  }
+  return nullptr;
+}
+
 DMatrix* DMatrix::Load(const std::string& uri,
                        bool silent,
                        bool load_row_split,
-                       const std::string& file_format,
-                       const size_t page_size) {
+                       const std::string& file_format) {
   std::string fname, cache_file;
   size_t dlm_pos = uri.find('#');
   if (dlm_pos != std::string::npos) {
@@ -682,35 +714,34 @@ DMatrix* DMatrix::Load(const std::string& uri,
 
   // legacy handling of binary data loading
   if (file_format == "auto" && npart == 1) {
-    int magic;
-    std::unique_ptr<dmlc::Stream> fi(dmlc::Stream::Create(fname.c_str(), "r", true));
-    if (fi != nullptr) {
-      common::PeekableInStream is(fi.get());
-      if (is.PeekRead(&magic, sizeof(magic)) == sizeof(magic)) {
-        if (!DMLC_IO_NO_ENDIAN_SWAP) {
-          dmlc::ByteSwap(&magic, sizeof(magic), 1);
-        }
-        if (magic == data::SimpleDMatrix::kMagic) {
-          DMatrix* dmat = new data::SimpleDMatrix(&is);
-          if (!silent) {
-            LOG(CONSOLE) << dmat->Info().num_row_ << 'x' << dmat->Info().num_col_ << " matrix with "
-              << dmat->Info().num_nonzero_ << " entries loaded from " << uri;
-          }
-          return dmat;
-        }
-      }
+    DMatrix *loaded = TryLoadBinary(fname, silent);
+    if (loaded) {
+      return loaded;
     }
   }
 
-  std::unique_ptr<dmlc::Parser<uint32_t> > parser(
-      dmlc::Parser<uint32_t>::Create(fname.c_str(), partid, npart, file_format.c_str()));
-  data::FileAdapter adapter(parser.get());
   DMatrix* dmat {nullptr};
-
   try {
-    dmat = DMatrix::Create(&adapter, std::numeric_limits<float>::quiet_NaN(), 1,
-                           cache_file, page_size);
-  } catch (dmlc::Error& e) {
+    if (cache_file.empty()) {
+      std::unique_ptr<dmlc::Parser<uint32_t>> parser(
+          dmlc::Parser<uint32_t>::Create(fname.c_str(), partid, npart,
+                                         file_format.c_str()));
+      data::FileAdapter adapter(parser.get());
+      dmat = DMatrix::Create(&adapter, std::numeric_limits<float>::quiet_NaN(),
+                             1, cache_file);
+    } else {
+      data::FileIterator iter{fname, uint32_t(partid), uint32_t(npart),
+                              file_format};
+      dmat = new data::SparsePageDMatrix{
+          &iter,
+          iter.Proxy(),
+          data::fileiter::Reset,
+          data::fileiter::Next,
+          std::numeric_limits<float>::quiet_NaN(),
+          1,
+          cache_file};
+    }
+  } catch (dmlc::Error &e) {
     std::vector<std::string> splited = common::Split(fname, '#');
     std::vector<std::string> args = common::Split(splited.front(), '?');
     std::string format {file_format};
@@ -734,10 +765,6 @@ DMatrix* DMatrix::Load(const std::string& uri,
     LOG(FATAL) << "Encountered parser error:\n" << e.what();
   }
 
-  if (!silent) {
-    LOG(CONSOLE) << dmat->Info().num_row_ << 'x' << dmat->Info().num_col_ << " matrix with "
-                 << dmat->Info().num_nonzero_ << " entries loaded from " << uri;
-  }
   /* sync up number of features after matrix loaded.
    * partitioned data will fail the train/val validation check
    * since partitioned data not knowing the real number of features. */
@@ -769,12 +796,19 @@ DMatrix *DMatrix::Create(DataIterHandle iter, DMatrixHandle proxy,
                          XGDMatrixCallbackNext *next, float missing,
                          int nthread,
                          int max_bin) {
-#if defined(XGBOOST_USE_CUDA)
-  return new data::IterativeDeviceDMatrix(iter, proxy, reset, next, missing, nthread, max_bin);
-#else
-  common::AssertGPUSupport();
-  return nullptr;
-#endif
+  return new data::IterativeDeviceDMatrix(iter, proxy, reset, next, missing,
+                                          nthread, max_bin);
+}
+
+template <typename DataIterHandle, typename DMatrixHandle,
+          typename DataIterResetCallback, typename XGDMatrixCallbackNext>
+DMatrix *DMatrix::Create(DataIterHandle iter, DMatrixHandle proxy,
+                         DataIterResetCallback *reset,
+                         XGDMatrixCallbackNext *next, float missing,
+                         int32_t n_threads,
+                         std::string cache) {
+  return new data::SparsePageDMatrix(iter, proxy, reset, next, missing, n_threads,
+                                     cache);
 }
 
 template DMatrix *DMatrix::Create<DataIterHandle, DMatrixHandle,
@@ -783,46 +817,42 @@ template DMatrix *DMatrix::Create<DataIterHandle, DMatrixHandle,
     XGDMatrixCallbackNext *next, float missing, int nthread,
     int max_bin);
 
+template DMatrix *DMatrix::Create<DataIterHandle, DMatrixHandle,
+                                  DataIterResetCallback, XGDMatrixCallbackNext>(
+    DataIterHandle iter, DMatrixHandle proxy, DataIterResetCallback *reset,
+    XGDMatrixCallbackNext *next, float missing, int32_t n_threads, std::string);
+
 template <typename AdapterT>
 DMatrix* DMatrix::Create(AdapterT* adapter, float missing, int nthread,
-                         const std::string& cache_prefix,  size_t page_size) {
-  if (cache_prefix.length() == 0) {
-    // Data split mode is fixed to be row right now.
-    return new data::SimpleDMatrix(adapter, missing, nthread);
-  } else {
-#if DMLC_ENABLE_STD_THREAD
-    return new data::SparsePageDMatrix(adapter, missing, nthread, cache_prefix,
-                                       page_size);
-#else
-    LOG(FATAL) << "External memory is not enabled in mingw";
-    return nullptr;
-#endif  // DMLC_ENABLE_STD_THREAD
-  }
+                         const std::string& cache_prefix) {
+  return new data::SimpleDMatrix(adapter, missing, nthread);
 }
 
 template DMatrix* DMatrix::Create<data::DenseAdapter>(
     data::DenseAdapter* adapter, float missing, int nthread,
-    const std::string& cache_prefix, size_t page_size);
+    const std::string& cache_prefix);
+template DMatrix* DMatrix::Create<data::ArrayAdapter>(
+    data::ArrayAdapter* adapter, float missing, int nthread,
+    const std::string& cache_prefix);
 template DMatrix* DMatrix::Create<data::CSRAdapter>(
     data::CSRAdapter* adapter, float missing, int nthread,
-    const std::string& cache_prefix, size_t page_size);
+    const std::string& cache_prefix);
 template DMatrix* DMatrix::Create<data::CSCAdapter>(
     data::CSCAdapter* adapter, float missing, int nthread,
-    const std::string& cache_prefix, size_t page_size);
+    const std::string& cache_prefix);
 template DMatrix* DMatrix::Create<data::DataTableAdapter>(
     data::DataTableAdapter* adapter, float missing, int nthread,
-    const std::string& cache_prefix, size_t page_size);
+    const std::string& cache_prefix);
 template DMatrix* DMatrix::Create<data::FileAdapter>(
     data::FileAdapter* adapter, float missing, int nthread,
-    const std::string& cache_prefix, size_t page_size);
+    const std::string& cache_prefix);
 template DMatrix* DMatrix::Create<data::CSRArrayAdapter>(
     data::CSRArrayAdapter* adapter, float missing, int nthread,
-    const std::string& cache_prefix, size_t page_size);
+    const std::string& cache_prefix);
 template DMatrix *
 DMatrix::Create(data::IteratorAdapter<DataIterHandle, XGBCallbackDataIterNext,
                                       XGBoostBatchCSR> *adapter,
-                float missing, int nthread, const std::string &cache_prefix,
-                size_t page_size);
+                float missing, int nthread, const std::string &cache_prefix);
 
 SparsePage SparsePage::GetTranspose(int num_columns) const {
   SparsePage transpose;
@@ -850,8 +880,15 @@ SparsePage SparsePage::GetTranspose(int num_columns) const {
           tid);
     }
   });
+
+  if (this->data.Empty()) {
+    transpose.offset.Resize(num_columns + 1);
+    transpose.offset.Fill(0);
+  }
+  CHECK_EQ(transpose.offset.Size(), num_columns + 1);
   return transpose;
 }
+
 void SparsePage::Push(const SparsePage &batch) {
   auto& data_vec = data.HostVector();
   auto& offset_vec = offset.HostVector();
@@ -907,7 +944,7 @@ uint64_t SparsePage::Push(const AdapterBatchT& batch, float missing, int nthread
   const size_t thread_size = batch_size / nthread;
 
   builder.InitBudget(expected_rows, nthread);
-  std::vector<std::vector<uint64_t>> max_columns_vector(nthread);
+  std::vector<std::vector<uint64_t>> max_columns_vector(nthread, std::vector<uint64_t>{0});
   dmlc::OMPException exec;
   std::atomic<bool> valid{true};
   // First-pass over the batch counting valid elements
@@ -917,7 +954,6 @@ uint64_t SparsePage::Push(const AdapterBatchT& batch, float missing, int nthread
       int tid = omp_get_thread_num();
       size_t begin = tid*thread_size;
       size_t end = tid != (nthread-1) ? (tid+1)*thread_size : batch_size;
-      max_columns_vector[tid].resize(1, 0);
       uint64_t& max_columns_local = max_columns_vector[tid][0];
 
       for (size_t i = begin; i < end; ++i) {
@@ -983,6 +1019,7 @@ void SparsePage::PushCSC(const SparsePage &batch) {
   auto const& other_offset = batch.offset.ConstHostVector();
 
   if (other_data.empty()) {
+    self_offset = other_offset;
     return;
   }
   if (!self_data.empty()) {
@@ -1037,8 +1074,12 @@ void SparsePage::PushCSC(const SparsePage &batch) {
 template uint64_t
 SparsePage::Push(const data::DenseAdapterBatch& batch, float missing, int nthread);
 template uint64_t
+SparsePage::Push(const data::ArrayAdapterBatch& batch, float missing, int nthread);
+template uint64_t
 SparsePage::Push(const data::CSRAdapterBatch& batch, float missing, int nthread);
 template uint64_t
+SparsePage::Push(const data::CSRArrayAdapterBatch& batch, float missing, int nthread);
+template uint64_t
 SparsePage::Push(const data::CSCAdapterBatch& batch, float missing, int nthread);
 template uint64_t
 SparsePage::Push(const data::DataTableAdapterBatch& batch, float missing, int nthread);
@@ -1049,5 +1090,6 @@ namespace data {
 
 // List of files that will be force linked in static links.
 DMLC_REGISTRY_LINK_TAG(sparse_page_raw_format);
+DMLC_REGISTRY_LINK_TAG(gradient_index_format);
 }  // namespace data
 }  // namespace xgboost
