@@ -238,6 +238,10 @@ void GenericParameter::ConfigureGpuId(bool require_gpu) {
 #endif  // defined(XGBOOST_USE_CUDA)
 }
 
+int32_t GenericParameter::Threads() const {
+  return common::OmpGetNumThreads(nthread);
+}
+
 using LearnerAPIThreadLocalStore =
     dmlc::ThreadLocalStore<std::map<Learner const *, XGBAPIThreadLocalEntry>>;
 
@@ -302,7 +306,6 @@ class LearnerConfiguration : public Learner {
     auto initialized = generic_parameters_.GetInitialised();
     auto old_seed = generic_parameters_.seed;
     generic_parameters_.UpdateAllowUnknown(args);
-    generic_parameters_.CheckDeprecated();
 
     ConsoleLogger::Configure(args);
     common::OmpSetNumThreads(&generic_parameters_.nthread);
@@ -565,9 +568,9 @@ class LearnerConfiguration : public Learner {
       ss << "\"" << diff.back() << "\"";
       ss << R"W( } might not be used.
 
-  This may not be accurate due to some parameters are only used in language bindings but
-  passed down to XGBoost core.  Or some parameters are not used but slip through this
-  verification. Please open an issue if you find above cases.
+  This could be a false alarm, with some parameters getting used by language bindings but
+  then being mistakenly passed down to XGBoost core, or some parameter actually being used
+  but getting flagged wrongly here. Please open an issue if you find any such cases.
 
 )W";
       LOG(WARNING) << ss.str();
@@ -1021,20 +1024,37 @@ class LearnerImpl : public LearnerIO {
   Learner *Slice(int32_t begin_layer, int32_t end_layer, int32_t step,
                  bool *out_of_bound) override {
     this->Configure();
+    CHECK_NE(this->learner_model_param_.num_feature, 0);
     CHECK_GE(begin_layer, 0);
     auto *out_impl = new LearnerImpl({});
+    out_impl->learner_model_param_ = this->learner_model_param_;
+    out_impl->generic_parameters_ = this->generic_parameters_;
     auto gbm = std::unique_ptr<GradientBooster>(GradientBooster::Create(
-        this->tparam_.booster, &this->generic_parameters_,
-        &this->learner_model_param_));
+        this->tparam_.booster, &out_impl->generic_parameters_,
+        &out_impl->learner_model_param_));
     this->gbm_->Slice(begin_layer, end_layer, step, gbm.get(), out_of_bound);
     out_impl->gbm_ = std::move(gbm);
+
     Json config { Object() };
     this->SaveConfig(&config);
     out_impl->mparam_ = this->mparam_;
     out_impl->attributes_ = this->attributes_;
-    out_impl->learner_model_param_ = this->learner_model_param_;
+    out_impl->SetFeatureNames(this->feature_names_);
+    out_impl->SetFeatureTypes(this->feature_types_);
     out_impl->LoadConfig(config);
     out_impl->Configure();
+    CHECK_EQ(out_impl->learner_model_param_.num_feature, this->learner_model_param_.num_feature);
+    CHECK_NE(out_impl->learner_model_param_.num_feature, 0);
+
+    auto erase_attr = [&](std::string attr) {
+      // Erase invalid attributes.
+      auto attr_it = out_impl->attributes_.find(attr);
+      if (attr_it != out_impl->attributes_.cend()) {
+        out_impl->attributes_.erase(attr_it);
+      }
+    };
+    erase_attr("best_iteration");
+    erase_attr("best_score");
     return out_impl;
   }
 
@@ -1194,6 +1214,12 @@ class LearnerImpl : public LearnerIO {
     *out_preds = &out_predictions.predictions;
   }
 
+  void CalcFeatureScore(std::string const& importance_type, common::Span<int32_t const> trees,
+                        std::vector<bst_feature_t>* features, std::vector<float>* scores) override {
+    this->Configure();
+    gbm_->FeatureScore(importance_type, trees, features, scores);
+  }
+
   const std::map<std::string, std::string>& GetConfigurationArguments() const override {
     return cfg_;
   }
